@@ -22,3 +22,9 @@ extern void
 
 extern char
 	*sprint proto((const char *, ...));
+
+extern bool jdebug;
+extern const char *jdpath;
+extern void jdprintf proto((const char *, ...));
+
+#define jdbg if (!jdebug) ; else jdprintf
