@@ -352,6 +352,18 @@ unmodify()
 	if (curbuf->b_modified) {
 		UpdModLine = YES;
 		curbuf->b_modified = NO;
+#ifdef RECOVER
+		/*
+		 * When a buffer transitions to unmodified
+		 * (most likely because it was saved, or if
+		 * the user invoked NotModified), then it
+		 * seems right to force a SyncRec (on the next
+		 * getch) so that any subsequent crash/recover
+		 * does not suggest recovery of this buffer.
+		 */
+		if (curbuf->b_type != B_SCRATCH)
+			ModCount = SyncFreq;
+#endif
 	}
 }
 
@@ -790,33 +802,6 @@ int errnum;
 }
 #endif /* NO_STRERROR */
 
-#ifdef NO_BSEARCH
-UnivPtr
-jbsearch(key, base, nmemb, size, compar)
-UnivConstPtr key, base;
-size_t nmemb, size;
-int (*compar) ptrproto((UnivConstPtr, UnivConstPtr));
-{
-	size_t n = nmemb;
-	const char *midp, *lop = (const char *) base;
-	int v;
-	while (n > 0) {
-		midp = lop + (n >> 1)*size;
-		v = (*compar)(key, midp);
-		if (debug) printf("%zu %zu %zu %d\n", (lop - (const char *)base)/size, (midp - (const char *)base)/size, n >> 1, v);
-		if (v == 0) {
-			return (UnivPtr)midp;
-		}
-		if (v > 0) {
-		    lop = midp + size;
-		    n--;
-		}
-		n >>= 1;
-	}
-	return NULL;
-}
-#endif /* NO_BSEARCH */
-
 /* decode a pair of characters representing \x or ^x */
 
 ZXchar
